package com.axiante.samples.stream;

import java.io.IOException;
import java.io.InputStream;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

@EqualsAndHashCode(callSuper = false)
@Slf4j

public class DbStream extends InputStream {
    @Getter
    ResultSet table;
    private AtomicInteger bufferIndex = new AtomicInteger(-1);;
    private char[] buffer;
    private boolean headers = true;
    ResultSetMetaData meta = null;
    
    public DbStream(@NonNull ResultSet table) {
	this.table = table;
	try {
	    meta =table.getMetaData();
	} catch (SQLException e) {
	    log.error("error setting source", e);
	}
    }

    @Override
    public int read() throws IOException {
	if (buffer != null && bufferIndex.get() < buffer.length - 1) {
	    return this.buffer[bufferIndex.incrementAndGet()];
	} else {
	    // fill the buffer with a new chunk
	    bufferIndex.set(-1);
	    try {
		nextBuffer();
	    } catch (Exception e) {
		// wrap into ioexception
		throw new IOException("Error retrieving next data", e);
	    }
	    if (this.buffer != null) {
		return this.buffer[bufferIndex.incrementAndGet()];
	    } else {
		return -1;
	    }
	}
    }

    private void nextBuffer() throws Exception {
	StringBuilder data = new StringBuilder();
	
	if (headers) {
	    // read the headers ...
	    headers = false;
	    data.append("\"").append((IntStream.rangeClosed(1, meta.getColumnCount()).mapToObj(i -> {
		try {
		    return meta.getColumnLabel(i);
		} catch (SQLException e) {
		    log.error("error getting column name for index " + i, e);
		}
		return null;
	    }).filter(Objects::nonNull).collect(Collectors.joining("\",\""))));
	} else {
	    if (table.next()) {
		data.append("\"").append(IntStream.rangeClosed(1, meta.getColumnCount()).mapToObj(i -> {
		    try {
			return table.getObject(i);
		    } catch (SQLException e) {
			log.error("error reading data at index " + i, e);
		    }
		    return null;
		}).filter(Objects::nonNull).map(Object::toString).collect(Collectors.joining("\",\"")));
	    } 
	}
	if ( data.length()>0) {
	    data.append("\"\n");
	    this.buffer = new char[data.length()];
	    data.getChars(0, data.length(), this.buffer, 0);
	} else {
	    this.buffer = null;
	}
    }

//    @Override
//    public int read(byte b[], int off, int len) throws IOException {
//	if (b == null) {
//	    throw new NullPointerException();
//	} else if (off < 0 || len < 0 || len > b.length - off) {
//	    throw new IndexOutOfBoundsException();
//	} else if (len == 0) {
//	    return 0;
//	}
//
//	int c = read();
//	if (c == -1) {
//	    return -1;
//	}
//	b[off] = (byte) c;
//
//	int i = 1;
//	try {
//	    for (; i < len; i++) {
//		c = read();
//		if (c == -1) {
//		    break;
//		} else if (c < 32 || c > 255) {
//		    log.debug("skipping control char " + c);
//		    continue;
//		} else {
//		    b[off + i] = (byte) c;
//		}
//	    }
//	} catch (IOException ee) {
//	    log.error("error reading stream ", ee);
//	}
//	return i;
//    }

}
