package com.axiante.samples.stream.utils;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.concurrent.atomic.AtomicBoolean;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class DerbyManager {
    private static final String dbName = "test";
    private static String dbURL = "jdbc:derby:memory:test";
    private static String tableName = "restaurants";
    // jdbc Connection
    private static Connection conn = null;
    private static Statement stmt = null;

    private static final AtomicBoolean started = new AtomicBoolean(false);

    public static void start() {
	synchronized (started) {
	    if (!started.getAndSet(true)) {
		createConnection();
		createTable();
	    }
	}
    }

    public static void stop() {
	synchronized (started) {
	    if (started.getAndSet(false)) {
		shutdown();
	    }
	}
    }

    private static void createConnection() {
	try {
	    Class.forName("org.apache.derby.jdbc.EmbeddedDriver").newInstance();
	    // Get a connection
	    conn = DriverManager.getConnection(dbURL + ";create=true");
	} catch (Exception except) {
	    log.error("error creating connection", except);
	}
    }

    private static void createTable() {
	try {
	    stmt = conn.createStatement();
	    stmt.execute("create table " + tableName + " (id integer, name varchar(255), city varchar(255))");
	    stmt.close();
	} catch (SQLException sqlExcept) {
	    log.error("error inserting data ", sqlExcept);
	}
	
    }
    public static void insertRestaurants(int id, String restName, String cityName) {
	try {
	    stmt = conn.createStatement();
	    stmt.execute("insert into " + tableName + " values (" + id + ",'" + restName + "','" + cityName + "')");
	    stmt.close();
	} catch (SQLException sqlExcept) {
	    log.error("error inserting data ", sqlExcept);
	}
    }

    public static ResultSet selectRestaurants() {
	try {
	    stmt = conn.createStatement();
	    return stmt.executeQuery("select * from " + tableName);
	} catch (SQLException sqlExcept) {
	    sqlExcept.printStackTrace();
	}
	return null;
    }

    private static void shutdown() {
	try {
	    if (stmt != null) {
		stmt.close();
	    }
	    if (conn != null) {
		DriverManager.getConnection(dbURL + ";drop=true");
		conn.close();
	    }
	} catch (SQLException sqlExcept) {
	    log.error("error shutting down server");
	}

    }
}
